﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Linq;

namespace U2UConsult.L2S.Linq
{
    public class Program
    {
        static void Main(string[] args)
        {
            // L2SOuterJoin();
            L2SOuterJoinWithProjection();

            Console.ReadKey();
        }

        /// <summary>
        /// Person-with-phones as a Data Transfer Object.
        /// </summary>
        private class PersonWithPhonesDto
        {
            public string LastName { get; set; }
            public IEnumerable<PersonPhone> PersonPhones { get; set; }
        }

        /// <summary>
        /// Person-with-phone as a Data Transfer Object.
        /// </summary>
        private class PersonWithPhoneDto
        {
            public string LastName { get; set; }

            public string PhoneNumber { get; set; }
        }

        public static void L2SOuterJoin()
        {
            List<PersonWithPhonesDto> query;

            using (AdventureWorks2008DataContext model = new AdventureWorks2008DataContext())
            {
                // Has no impact on the query:
                // DataLoadOptions dlo = new DataLoadOptions();
                // dlo.LoadWith<Person>(p => p.PersonPhones);
                // model.LoadOptions = dlo;

                query = (from p in model.Persons
                         // Has no impact on the query:
                         // join pp in model.PersonPhones
                         // on p.BusinessEntityID equals pp.BusinessEntityID
                         // into phones
                         // select new { p.LastName, PersonPhones = phones };
                         select new PersonWithPhonesDto() { LastName = p.LastName, PersonPhones = p.PersonPhones }).ToList();
            }

            foreach (var person in query)
            {
                Console.WriteLine(string.Format("{0}: {1}", person.LastName, person.PersonPhones.First().PhoneNumber));
            }

            // Only in VS Ultimate edition:
            // Put a breakpoint on the next curly brace, and see the SQL in IntelliTrace
        }

        public static void L2SOuterJoinWithProjection()
        {
            List<PersonWithPhoneDto> query;

            using (AdventureWorks2008DataContext model = new AdventureWorks2008DataContext())
            {
                query = (from p in model.Persons
                         from pp in
                             (from zz in model.PersonPhones
                              where p.BusinessEntityID == zz.BusinessEntityID
                              select new { zz.PhoneNumber }).DefaultIfEmpty()
                         select new PersonWithPhoneDto() { LastName = p.LastName, PhoneNumber = pp.PhoneNumber }).ToList();
            }

            foreach (var person in query)
            {
                Console.WriteLine(string.Format("{0}: {1}", person.LastName, person.PhoneNumber));
            }

            // Only in VS Ultimate edition:
            // Put a breakpoint on the next curly brace, and see the SQL in IntelliTrace
        }
    }
}
